@@ -195,7 +195,8 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
       fputs ("  rmdup\n",stdout);
 
       fputs (_("Per-Line operations:\n"),stdout);
-      fputs ("  base64, debase64, md5, sha1, sha256, sha512,\n", stdout);
+      fputs ("  base64, debase64, md5, sha1, sha224, sha256, sha384, sha512,\n",
+             stdout);
       fputs ("  bin, strbin, round, floor, ceil, trunc, frac,\n", stdout);
       fputs ("  dirname, basename, barename, extname, getnum, cut\n", stdout);
 
@@ -208,8 +209,8 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
 
       fputs (_("Statistical Grouping operations:\n"),stdout);
       fputs ("\
-  mean, trimmean, median, q1, q3, iqr, perc, mode, antimode, \n\
-  pstdev, sstdev, pvar, svar, mad, madraw,\n\
+  mean, geomean, harmmean, trimmean, median, q1, q3, iqr, perc,\n\
+  mode, antimode, pstdev, sstdev, pvar, svar, mad, madraw,\n\
   pskew, sskew, pkurt, skurt, dpo, jarque,\n\
   scov, pcov, spearson, ppearson\n\
 \n", stdout);
