@@ -18,7 +18,6 @@
 #include <getopt.h>
 #include <limits.h>
 #include <inttypes.h>
-#include <intprops.h>
 #include <sys/wait.h>
 
 #include "system.h"
@@ -27,7 +26,6 @@
 #include "closeout.h"
 #include "linebuffer.h"
 #include "die.h"
-#include "error.h"
 #include "quote.h"
 #include "key-compare.h"
 #define Version VERSION
@@ -42,6 +40,24 @@
 #define AUTHORS \
   proper_name ("Assaf Gordon")
 
+static bool debug_print_pid = false;
+
+static void _GL_ATTRIBUTE_FORMAT ((__printf__, 1, 2))
+dbg_printf (char const *msg, ...)
+{
+  va_list args;
+  /* TODO: use gnulib's 'program_name' instead?  */
+  if (debug_print_pid)
+    fprintf (stderr, "decorate (PID %i):", (int)getpid ());
+  else
+    fputs ("decorate: ", stderr);
+
+  va_start (args, msg);
+  vfprintf (stderr, msg, args);
+  va_end (args);
+  fputc ('\n', stderr);
+}
+
 /* Until someone better comes along */
 const char version_etc_copyright[] = "Copyright %s %d Assaf Gordon" ;
 
@@ -57,6 +73,9 @@ static char **sort_extra_args = NULL;
 static size_t sort_extra_args_used = 0;
 static size_t sort_extra_args_allocated = 0;
 
+static long skip_header_lines = 0 ;
+static FILE* header_out = NULL ;
+
 enum
 {
   DEBUG_OPTION = CHAR_MAX + 1,
@@ -69,6 +88,7 @@ enum
   CHECK_OPTION,
   NMERGE_OPTION,
   RANDOM_SOURCE_OPTION,
+  HEADER_OPTION,
   PARALLEL_OPTION
 };
 
@@ -80,6 +100,7 @@ static struct option const longopts[] =
   {"-debug", no_argument, NULL, DEBUG_OPTION},
   {"zero-terminated", no_argument, NULL, 'z'},
   {"print-sort-args", no_argument, NULL, PRINT_SORT_ARGS_OPTION},
+  {"header", required_argument, NULL, HEADER_OPTION},
 
   /* sort options, passed as-is to the sort process */
   {"check", optional_argument, NULL, CHECK_OPTION},
@@ -106,7 +127,7 @@ add_sort_extra_args (char* s)
     sort_extra_args = x2nrealloc (sort_extra_args,
                                   &sort_extra_args_allocated,
                                   sizeof *sort_extra_args);
-  sort_extra_args[sort_extra_args_used++] = xstrdup(s);
+  sort_extra_args[sort_extra_args_used++] = xstrdup (s);
 }
 
 void
@@ -117,10 +138,9 @@ usage (int status)
   else
     {
       printf (_("\
-Usage:\n\
-  %s [OPTION]... [INPUT]\n\
-  %s --decorate [OPTION]... [INPUT]\n\
-  %s --undecorate N [OPTION]... [INPUT]\n\
+Usage: %s [OPTION]... [INPUT]\n\
+   or: %s --decorate [OPTION]... [INPUT]\n\
+   or: %s --undecorate N [OPTION]... [INPUT]\n\
 \n\
 "),
               program_name, program_name, program_name);
@@ -128,52 +148,59 @@ Usage:\n\
 Converts (and optionally sorts) fields of various formats\n\
 "), stdout);
 
-     putchar('\n');
+     putchar ('\n');
 
      fputs (_("\
-  1. without --decorate and --undecorate: runs sort(1)\n\
-     automatically and outputs the sorted data.\n\
-     It is the easiest method to use.\n\
+With --decorate: adds the converted fields to the start\n\
+of each line and prints and prints it to STDOUT; does not sort.\n\
 "), stdout);
 
+     putchar ('\n');
+
      fputs (_("\
-  2. with --decorate: adds the converted fields to the start\n\
-     of each line and prints and prints it to STDOUT. sort(1) can then be\n\
-     used on the output.\n\
-     Use --print-sort-args to see the recommended arguments.\n\
+With --undecorate: removes the first N fields from the input;\n\
+Use as post-processing step after sort(1).\n\
 "), stdout);
 
+     putchar ('\n');
+
      fputs (_("\
-  3. with --undecorate: removes the first N fields from the\n\
-     input. It can be used as post-processing step after sort(1).\n\
+Without --decorate and --undecorate: automatically decorates the input,\n\
+runs sort(1) and undecorates the result; This is the easiest method to use.\n\
 "), stdout);
 
-     putchar('\n');
+     putchar ('\n');
 
      fputs (_("\
-OPTIONS:\n\
+General Options:\n\
 "), stdout);
 
 
      fputs (_("\
       --decorate             decorate/convert the specified fields and print\n\
                              the output to STDOUT. Does not automatically run\n\
-                             sort(1) or undecorates the output.\n\
+                             sort(1) or undecorates the output\n\
+"), stdout);
+     fputs (_("\
+      --header=N             does not decorate or sort the first N lines\n\
+  -H                         same as --header=N\n\
 "), stdout);
       fputs (_("\
   -k, --key=KEYDEF           key/field to sort; same syntax as sort(1),\n\
                              optionally followed by ':method' to convert\n\
-                             to the field into a sortable value.\n\
-                             see below for details and examples.\n\
+                             to the field into a sortable value; see examples\n\
+                             and available conversion below\n\
 "), stdout);
      fputs (_("\
   -t, --field-separator=SEP  use SEP instead of non-blank to blank transition\n\
 "), stdout);
      fputs (_("\
-      --print-sort-args      print the adjusted parameters for sort(1)\n\
+      --print-sort-args      print adjusted parameters for sort(1); Useful\n\
+                             when using --decorate and then manually running\n\
+                             sort(1)\n\
 "), stdout);
      fputs (_("\
-      --undecorate=N         removes the first N fields.\n\
+      --undecorate=N         removes the first N fields\n\
 "), stdout);
       fputs (_("\
   -z, --zero-terminated      line delimiter is NUL, not newline\n\
@@ -181,13 +208,13 @@ OPTIONS:\n\
      fputs (HELP_OPTION_DESCRIPTION, stdout);
      fputs (VERSION_OPTION_DESCRIPTION, stdout);
 
-     putchar('\n');
+     putchar ('\n');
 
      fputs (_("\
 The following options are passed to sort(1) as-is:\n\
 "), stdout);
 
-     putchar('\n');
+     putchar ('\n');
 
      fputs (_("\
   -c, --check\n\
@@ -202,31 +229,49 @@ The following options are passed to sort(1) as-is:\n\
 "), stdout);
 
 
-
-     putchar('\n');
+     putchar ('\n');
 
      fputs (_("\
-Example: the following two commands are equivalent:\n\
+Available conversions methods (use with -k):\n\
 "), stdout);
 
-     putchar('\n');
+     putchar ('\n');
 
-     printf("   %s -k2,2:ipv4 -k3,3nr FILE.TXT\n", program_name);
+     for (int i = 0 ; builtin_conversions[i].name; ++i)
+       printf ("  %-10s   %s\n", builtin_conversions[i].name,
+               builtin_conversions[i].description);
 
-     putchar('\n');
+     putchar ('\n');
 
-     printf("   %s --decorate -k2,2:ipv4 FILE.TXT | sort -k1,1 -k4,4nr \\\n \
+     fputs (_("\
+Examples:\n\
+"), stdout);
+
+     putchar ('\n');
+
+     fputs (_("\
+The following two invocations are equivalent:\n\
+"), stdout);
+
+     putchar ('\n');
+
+     printf ("   %s -k2,2:ipv4 -k3,3nr FILE.TXT\n", program_name);
+
+     putchar ('\n');
+
+     printf ("   %s --decorate -k2,2:ipv4 FILE.TXT | sort -k1,1 -k4,4nr \\\n \
        | %s --undecorate 1\n", program_name, program_name);
 
-     putchar('\n');
+     putchar ('\n');
 
      fputs (_("\
-Example: decorated output of roman numerals:\n\
+Decorated output of roman numerals:\n\
 "), stdout);
 
-     putchar('\n');
+     putchar ('\n');
 
-     printf("  $ printf \"%%s\\n\" C V III IX XI | %s -k1,1:roman --decorate\n", program_name);
+     printf ("  $ printf \"%%s\\n\" C V III IX XI | "
+             "%s -k1,1:roman --decorate\n", program_name);
      fputs ("\
   0000100 C\n\
   0000005 V\n\
@@ -236,31 +281,25 @@ Example: decorated output of roman numerals:\n\
 ", stdout);
 
 
-     putchar('\n');
+     putchar ('\n');
 
-     fputs (_("\
-Available conversions methods:\n\
-"), stdout);
-     for (int i = 0 ; builtin_conversions[i].name; ++i)
-       printf ("  %-10s   %s\n", builtin_conversions[i].name,
-               builtin_conversions[i].description);
-     putchar('\n');
 
-      fputs (_("For detailed usage information and examples, see\n"),stdout);
-      printf ("  man %s\n", program_name);
-      fputs (_("The manual and more examples are available at\n"), stdout);
-      fputs ("  " PACKAGE_URL "\n\n", stdout);
+     fputs (_("For detailed usage information and examples, see\n"),stdout);
+     printf ("  man %s\n", program_name);
+     fputs (_("The manual and more examples are available at\n"), stdout);
+     fputs ("  " PACKAGE_URL "\n\n", stdout);
     }
   exit (status);
 }
 
-static void
+static bool
 decorate_fields (struct linebuffer *linebuf)
 {
+  bool ok = true;
   struct line l;   /* The struct used by key-compare */
   struct line *a;
 
-  size_t len = linebuf->length;
+  //size_t len = linebuf->length;
   struct keyfield *key = decorate_keylist;
 
   l.text = linebuf->buffer;
@@ -268,24 +307,26 @@ decorate_fields (struct linebuffer *linebuf)
   l.keybeg = NULL;
   l.keylim = NULL;
 
+
+
   l.keybeg = begfield (&l, decorate_keylist);
   l.keylim = limfield (&l, decorate_keylist);
 
   if (l.keybeg >= l.keylim)
     {
-      /* TODO: is this the correct way to detect 'no limit' ?
-       * (ie compare keys until the end of the line)*/
-      l.keylim = l.keybeg + len - (l.keybeg - l.text);
       if (l.length>0 && l.text[l.length-1]==eol_delimiter)
         {
           if (debug)
-            fprintf(stderr,"KO chomp");
+            dbg_printf ("chomp");
           l.length--;
-          l.keylim--;
         }
 
+      /* TODO: is this the correct way to detect 'no limit' ?
+       * (ie compare keys until the end of the line)*/
+      l.keylim = l.keybeg + l.length - (l.keybeg - l.text);
+
       if (debug)
-        fprintf(stderr,"YO keylim - keybeg = %zu\n", (l.keylim-l.keybeg));
+        dbg_printf ("keylim - keybeg = %zu", (l.keylim-l.keybeg));
     }
 
 
@@ -315,13 +356,13 @@ decorate_fields (struct linebuffer *linebuf)
       /* Process the extracted key in NUL-terminated string 'ta' */
 
       if (debug)
-        fprintf(stderr,"extracted field: '%s'\n", ta);
+        dbg_printf ("extracted field: '%s'", ta);
 
       if (key->decorate_fn)
         {
           /* Innternal conversions */
-          key->decorate_fn (ta);
-        }
+          ok = ok && key->decorate_fn (ta);
+         }
       else
         {
           /* run external command */
@@ -355,6 +396,7 @@ decorate_fields (struct linebuffer *linebuf)
             }
         }
     }
+  return ok;
 }
 
 static void
@@ -364,14 +406,14 @@ decorate_file (const char *infile)
   struct linebuffer lb;
 
   if (! (STREQ (infile, "-") || freopen (infile, "r", stdin)))
-    die (EXIT_FAILURE, errno, "%s", quotef (infile));
+    die (SORT_FAILURE, errno, "%s", quotef (infile));
 
   //fadvise (stdin, FADVISE_SEQUENTIAL);
 
   initbuffer (&lb);
 
   if (debug)
-    fprintf(stderr,"decorate: starting read loop\n");
+    dbg_printf ("starting decoreate_file read loop");
 
   while (!feof (stdin))
     {
@@ -380,24 +422,42 @@ decorate_file (const char *infile)
 
       ++linenum;
 
+      if (skip_header_lines)
+        {
+          if (debug)
+            dbg_printf ("skipping header line");
 
-      //fprintf(stderr,"before chomp: len %zu: '%.*s'\n", lb.length, (int)lb.length, lb.buffer);
-      decorate_fields (&lb);
+          if (fwrite (lb.buffer, 1, lb.length, header_out) != lb.length)
+            die (SORT_FAILURE, errno, _("fwrite (header line) failed"));
+
+          --skip_header_lines;
+
+          /* if no more header lines are left - flush the header lines,
+             to ensure they are passed to the output stream before all
+             other lines */
+          if ((skip_header_lines == 0) && fflush (header_out) == EOF)
+            die (SORT_FAILURE, errno, _("fflush (header line) failed"));
+
+          continue;
+        }
+
+      if (!decorate_fields (&lb))
+        die (SORT_FAILURE, errno, _("conversion failed in line %zu"), linenum);
 
       if (debug)
-        fprintf(stderr,"decorate: writing line: '%.*s'\n", (int)lb.length, lb.buffer);
+        dbg_printf ("writing line: '%.*s'", (int)lb.length, lb.buffer);
 
       if (fwrite (lb.buffer, 1, lb.length, stdout) != lb.length)
-        die (EXIT_FAILURE, errno, _("decorate: fwrite failed"));
+        die (SORT_FAILURE, errno, _("decorate: fwrite failed"));
 
     }
 
   // closefiles:
   if (ferror (stdin) || fclose (stdin) != 0)
-    die (EXIT_FAILURE, 0, _("error reading %s"), quoteaf (infile));
+    die (SORT_FAILURE, 0, _("error reading %s"), quoteaf (infile));
 
   if (debug)
-    fprintf(stderr,"decorate: end loop\n");
+    dbg_printf ("end decorate_file read loop");
 
   /* stdout is handled via the atexit-invoked close_stdout function.  */
   free (lb.buffer);
@@ -412,15 +472,15 @@ undecorate_file (const char *infile, int num_fields)
   struct linebuffer lb;
   struct keyfield key;
   struct line l;
-  memset (&l, 0, sizeof(l));
+  memset (&l, 0, sizeof (l));
 
   /* Create a dummy keyfield, encompasing the first N fields to skip */
-  memset(&key, 0, sizeof(key));
+  memset (&key, 0, sizeof (key));
   key.sword = num_fields;
   key.skipsblanks = true;
 
   if (! (STREQ (infile, "-") || freopen (infile, "r", stdin)))
-    die (EXIT_FAILURE, errno, "%s", quotef (infile));
+    die (SORT_FAILURE, errno, "%s", quotef (infile));
 
   //fadvise (stdin, FADVISE_SEQUENTIAL);
 
@@ -441,22 +501,22 @@ undecorate_file (const char *infile, int num_fields)
         --l.length;
 
       /* skip the first N fields */
-      char* p = begfield(&l, &key);
+      char* p = begfield (&l, &key);
       size_t t = l.length - (p-l.text);
 
       if (debug)
         {
-          fprintf(stderr,"Orig Line: %zu chars: '%.*s'\n", l.length, (int)l.length, l.text);
-          fprintf(stderr,"Undec Line: %zu chars: '%.*s'\n", t, (int)t, p);
+          dbg_printf ("input Line: %zu chars: '%.*s'", l.length,
+                     (int)l.length, l.text);
+          dbg_printf ("undecorated Line: %zu chars: '%.*s'", t, (int)t, p);
         }
       fwrite (p, 1, t, stdout);
 
       fputc (eol_delimiter, stdout);
     }
 
-  // closefiles:
   if (ferror (stdin) || fclose (stdin) != 0)
-    die (EXIT_FAILURE, 0, _("error reading %s"), quoteaf (infile));
+    die (SORT_FAILURE, 0, _("error reading %s"), quoteaf (infile));
 
   /* stdout is handled via the atexit-invoked close_stdout function.  */
   free (lb.buffer);
@@ -465,7 +525,7 @@ undecorate_file (const char *infile, int num_fields)
 
 /* this code was copied from src/sort.c */
 struct keyfield*
-parse_sort_key_arg(const char* optarg, char const** endpos)
+parse_sort_key_arg (const char* optarg, char const** endpos)
 {
   char const *s;
   struct keyfield key_buf;
@@ -532,7 +592,8 @@ parse_sort_key_arg(const char* optarg, char const** endpos)
 
 
 static void
-parse_external_conversion_spec (const char* optarg, const char *s, struct keyfield *key)
+parse_external_conversion_spec (const char* optarg, const char *s,
+                                struct keyfield *key)
 {
   ignore_value (key);
 
@@ -546,7 +607,8 @@ parse_external_conversion_spec (const char* optarg, const char *s, struct keyfie
 
 
 static void
-parse_builtin_conversion_spec (const char* optarg, const char *s, struct keyfield *key)
+parse_builtin_conversion_spec (const char* optarg, const char *s,
+                               struct keyfield *key)
 {
   bool found = false;
 
@@ -576,10 +638,11 @@ void check_allowed_key_flags (const struct keyfield* key)
       || (key->general_numeric) || (key->human_numeric)
       || (key->month) || (key->numeric)
       || (key->random) || (key->version))
-    badfieldspec (optarg, N_("ordering flags (b/d/i/h/n/g/M/R/V) can not be combined with a conversion function"));
+    badfieldspec (optarg, N_("ordering flags (b/d/i/h/n/g/M/R/V) can"
+                             "not be combined with a conversion function"));
 }
 
-void parse_key_arg(const char* optarg)
+void parse_key_arg (const char* optarg)
 {
   char const *s;
   struct keyfield *key = parse_sort_key_arg (optarg, &s);
@@ -628,7 +691,7 @@ insert_decorate_key (struct keyfield *key_arg)
 
 
 int
-adjust_key_fields()
+adjust_key_fields ()
 {
   struct keyfield *key = keylist;
   int cnt = 0;
@@ -640,7 +703,7 @@ adjust_key_fields()
   }  while (key && ((key = key->next)));
 
   if (debug)
-    fprintf(stderr, "Found %d decorate fields\n", cnt);
+    dbg_printf ("found %d decorated fields", cnt);
 
   key = keylist;
   do {
@@ -689,10 +752,10 @@ build_sort_process_args ()
 
   /* Step 2: allocate and build argv.
      The terminating NULL is implicit thanks to calloc. */
-  char** argv = xcalloc (argc, sizeof(char*));
+  char** argv = xcalloc (argc, sizeof (char*));
   int i = 0;
 
-  argv[i++] = xstrdup("sort"); /* argv[0] */
+  argv[i++] = xstrdup ("sort"); /* argv[0] */
 
   key = keylist;
   do {
@@ -709,7 +772,7 @@ build_sort_process_args ()
 
 
 static void
-do_decorate(int optind, int argc, char** argv)
+do_decorate (int optind, int argc, char** argv)
 {
   if (optind < argc)
     {
@@ -721,7 +784,7 @@ do_decorate(int optind, int argc, char** argv)
 }
 
 static void
-do_undecorate(int optind, int argc, char** argv, int undecorate_fields)
+do_undecorate (int optind, int argc, char** argv, int undecorate_fields)
 {
   if (optind < argc)
     {
@@ -737,6 +800,7 @@ int
 main (int argc, char **argv)
 {
   int opt;
+  char *endp;
   int undecorate_fields = 0;
   int num_decorate_fields = 0;
   bool decorate_only = false;
@@ -751,75 +815,88 @@ main (int argc, char **argv)
 
   //atexit (close_stdout);
 
-  while ((opt = getopt_long (argc, argv, "cCsS:T:uk:t:z", longopts, NULL)) != -1)
+  while ((opt = getopt_long (argc, argv, "cCHsS:T:uk:t:z",
+                             longopts, NULL)) != -1)
     {
       switch (opt)
         {
         case 's':
-          add_sort_extra_args("-s");
+          add_sort_extra_args ("-s");
           break;
 
         case 'u':
-          add_sort_extra_args("-u");
+          add_sort_extra_args ("-u");
           break;
 
         case 'S':
-          add_sort_extra_args("-S");
-          add_sort_extra_args(optarg);
+          add_sort_extra_args ("-S");
+          add_sort_extra_args (optarg);
           break;
 
         case 'T':
-          add_sort_extra_args("-T");
-          add_sort_extra_args(optarg);
+          add_sort_extra_args ("-T");
+          add_sort_extra_args (optarg);
           break;
 
         case 'k':
           parse_key_arg (optarg);
           break;
 
+        case 'H':
+          skip_header_lines = 1;
+          break;
+
+        case HEADER_OPTION:
+          errno = 0 ;
+          skip_header_lines = strtol (optarg, &endp, 10);
+          if (errno || (*endp != '\0'))
+            die (SORT_FAILURE, 0, _("invalid number of header lines %s"),
+                 quote (optarg));
+          break;
+
           /* pass the various check options as-is to sort - do not validate */
         case CHECK_OPTION:
           if (!optarg)
             {
-              add_sort_extra_args("--check");
+              add_sort_extra_args ("--check");
             }
           else
             {
-              char *p = xcalloc (8 + strlen(optarg) + 1, 1);
+              char *p = xcalloc (8 + strlen (optarg) + 1, 1);
               strcpy (p, "--check=");
               strcat (p, optarg);
-              add_sort_extra_args(p);
+              add_sort_extra_args (p);
               free (p);
             }
           break;
 
         case 'c':
-          add_sort_extra_args("-c");
+          add_sort_extra_args ("-c");
           break;
 
         case 'C':
-          add_sort_extra_args("-C");
+          add_sort_extra_args ("-C");
           break;
 
 
         case COMPRESS_PROGRAM_OPTION:
-          add_sort_extra_args("--compress-program");
-          add_sort_extra_args(optarg);
+          add_sort_extra_args ("--compress-program");
+          add_sort_extra_args (optarg);
           break;
 
         case RANDOM_SOURCE_OPTION:
-          add_sort_extra_args("--random-source");
-          add_sort_extra_args(optarg);
+          add_sort_extra_args ("--random-source");
+          add_sort_extra_args (optarg);
           break;
 
         case NMERGE_OPTION:
-          add_sort_extra_args("--batch-size");
-          add_sort_extra_args(optarg);
+          add_sort_extra_args ("--batch-size");
+          add_sort_extra_args (optarg);
           break;
 
         case PARALLEL_OPTION:
-          add_sort_extra_args("--parallel");
-          add_sort_extra_args(optarg);
+          add_sort_extra_args ("--parallel");
+          add_sort_extra_args (optarg);
           break;
 
         case 't':
@@ -847,12 +924,12 @@ main (int argc, char **argv)
             tab = newtab;
           }
 
-          add_sort_extra_args("-t");
-          add_sort_extra_args(optarg);
+          add_sort_extra_args ("-t");
+          add_sort_extra_args (optarg);
           break;
 
         case 'z':
-          add_sort_extra_args("-z");
+          add_sort_extra_args ("-z");
           eol_delimiter = '\0';
           break;
 
@@ -861,9 +938,11 @@ main (int argc, char **argv)
           break;
 
         case UNDECORATE_OPTION:
-          undecorate_fields = atoi(optarg);
-          if (undecorate_fields <= 0)
-            die (SORT_FAILURE, 0, _("invalid number of fields to undecorate %s"),
+          errno = 0 ;
+          undecorate_fields = strtol (optarg, &endp, 10);
+          if (errno || (*endp != '\0') || (undecorate_fields <= 0))
+            die (SORT_FAILURE, 0,
+                 _("invalid number of fields to undecorate %s"),
                  quote (optarg));
           break;
 
@@ -880,18 +959,21 @@ main (int argc, char **argv)
         case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
 
         default:
-          usage (EXIT_FAILURE);
+          usage (SORT_FAILURE);
         }
     }
 
   if (decorate_only && undecorate_fields)
-    die (SORT_FAILURE, 0, _("--decorate and --undecorate options are mutually exclusive"));
+    die (SORT_FAILURE, 0,
+         _("--decorate and --undecorate options are mutually exclusive"));
 
   if (undecorate_fields && keylist)
-    die (SORT_FAILURE, 0, _("--undecorate cannot be used with --keys or --decorate"));
+    die (SORT_FAILURE, 0,
+         _("--undecorate cannot be used with --keys or --decorate"));
 
   if (!keylist && !undecorate_fields)
-    die (SORT_FAILURE, 0, _("missing -k/--key decoration or --undecorate options"));
+    die (SORT_FAILURE, 0,
+         _("missing -k/--key decoration or --undecorate options"));
 
   if (keylist)
     {
@@ -899,7 +981,8 @@ main (int argc, char **argv)
         debug_keylist (stderr);
       num_decorate_fields = adjust_key_fields ();
       if (!num_decorate_fields)
-        die (SORT_FAILURE, 0, _("no decorated keys specified. use standard sort(1) instead"));
+        die (SORT_FAILURE, 0,
+             _("no decorated keys specified. use sort instead"));
       if (debug)
         debug_keylist (stderr);
     }
@@ -912,17 +995,18 @@ main (int argc, char **argv)
       while (*p)
         {
           /* TODO: shell quoting / escaping */
-          fputs(*p, stdout);
+          fputs (*p, stdout);
           ++p;
           if (*p)
-            fputc(' ', stdout);
+            fputc (' ', stdout);
         }
-      fputc('\n', stdout);
-      exit(EXIT_SUCCESS);
+      fputc ('\n', stdout);
+      exit (EXIT_SUCCESS);
     }
 
   if (decorate_only)
     {
+      header_out = stdout;
       do_decorate (optind, argc, argv);
     }
   else if (undecorate_fields)
@@ -932,25 +1016,22 @@ main (int argc, char **argv)
   else
     {
       /* decorate + sort + undecorate */
+      debug_print_pid = true;
+
       int dec_sort_fds[2];
       int sort_undec_fds[2];
-      if (pipe(dec_sort_fds) != 0)
+      if (pipe (dec_sort_fds) != 0)
         die (SORT_FAILURE, errno, _("failed to create dec-sort pipe"));
-      if (pipe(sort_undec_fds) != 0)
+      if (pipe (sort_undec_fds) != 0)
         die (SORT_FAILURE, errno, _("failed to create sort-undec pipe"));
 
       pid_t undec_pid = -1, sort_pid = -1;
-      if (debug)
-        fprintf(stderr,"=1= my PID = %i   undec_pid = %i, sort_pid = %i\n", getpid(), undec_pid, sort_pid);
 
       /* fork for the 'decorate' process */
       undec_pid = fork ();
       if (undec_pid == -1)
         die (SORT_FAILURE, errno, _("failed to fork-1"));
 
-      if (debug)
-        fprintf(stderr,"=2= my PID = %i   undec_pid = %i, sort_pid = %i\n", getpid(), undec_pid, sort_pid);
-
       /* if this is the parent, fork again for the sort process */
       if (undec_pid != 0)
         {
@@ -959,65 +1040,76 @@ main (int argc, char **argv)
             die (SORT_FAILURE, errno, _("failed to fork-2"));
         }
 
-      if (debug)
-        fprintf(stderr,"=3= my PID = %i   undec_pid = %i, sort_pid = %i\n", getpid(), undec_pid, sort_pid);
 
       if (undec_pid!=0 && sort_pid!=0)
         {
           if (debug)
-            fprintf(stderr,"DECORATE process starting (PID %i)\n", getpid());
+            dbg_printf ("main process starting\n");
 
           /* close the read-end pipe */
-          if (close(sort_undec_fds[0])!=0)
-            die (SORT_FAILURE, errno, _("failed to close sort-undec read-pipe"));
-          if (close(sort_undec_fds[1])!=0)
-            die (SORT_FAILURE, errno, _("failed to close sort-undec write-pipe"));
-          if (close(dec_sort_fds[0])!=0)
+          if (close (sort_undec_fds[0])!=0)
+            die (SORT_FAILURE, errno,
+                 _("failed to close sort-undec read-pipe"));
+          if (close (sort_undec_fds[1])!=0)
+            die (SORT_FAILURE, errno,
+                 _("failed to close sort-undec write-pipe"));
+          if (close (dec_sort_fds[0])!=0)
             die (SORT_FAILURE, errno, _("failed to close dec-sort read-pipe"));
 
+          /* If there are header lines to skip, they need to be printed
+             directly to our current STDOUT, not to the sort-pipe.
+             save STDOUT for later. */
+          if (skip_header_lines)
+            {
+              int fd = dup (STDOUT_FILENO);
+              if (fd==-1)
+                die (SORT_FAILURE, errno, _("failed to dup STDOUT fd"));
+              header_out = fdopen (fd, "w");
+              if (!header_out)
+                die (SORT_FAILURE, errno, _("fdopen (dup (stdout)) failed"));
+            }
+
           /* replace the STDOUT of this process, to the write-end pipe */
-          if (dup2(dec_sort_fds[1], STDOUT_FILENO) == -1)
+          if (dup2 (dec_sort_fds[1], STDOUT_FILENO) == -1)
             die (SORT_FAILURE, errno, _("failed to reassign dec-sort STDOUT"));
 
           do_decorate (optind, argc, argv);
 
-          if (debug)
-            fprintf(stderr,"decorate: closing STDOUT\n");
-
-          fclose(stdout);
-          close(dec_sort_fds[1]);
+          fclose (stdout);
+          close (dec_sort_fds[1]);
 
           if (debug)
-            fprintf(stderr,"decorate: waiting for children 1\n");
+            dbg_printf ("main process: waiting for children");
 
+          /* TODO: pass exit codes from sort/undecorate */
           waitpid (undec_pid, NULL, 0);
-
-          if (debug)
-            fprintf(stderr,"decorate: waiting for children 2\n");
-
           waitpid (sort_pid, NULL, 0);
 
           if (debug)
-            fprintf(stderr,"decorate: done\n");
+            dbg_printf ("main process: done");
         }
       else if (sort_pid==0)
         {
           /* This is the sort process */
           if (debug)
-            fprintf(stderr,"SORT child starting (pid %i)\n", getpid());
+            dbg_printf ("sort process starting");
 
-          if (close(dec_sort_fds[1])!=0)
+          if (close (dec_sort_fds[1])!=0)
             die (SORT_FAILURE, errno, _("failed to close dec-sort write-pipe"));
-          if (close(sort_undec_fds[0])!=0)
-            die (SORT_FAILURE, errno, _("failed to close sort-undec read-pipe"));
+          if (close (sort_undec_fds[0])!=0)
+            die (SORT_FAILURE, errno,
+                 _("failed to close sort-undec read-pipe"));
 
-          /* replace the STDIN of this process, to the read-end pipe of 'decorate' */
-          if (dup2(dec_sort_fds[0], STDIN_FILENO) == -1)
+          /* replace the STDIN of this process,
+             to the read-end pipe of 'decorate' */
+          if (dup2 (dec_sort_fds[0], STDIN_FILENO) == -1)
             die (SORT_FAILURE, errno, _("failed to reassign dec-sort STDIN"));
 
-          /* replace the STDOUT of this process, to the write-end pipe of 'undecorate' */
-          if (dup2(sort_undec_fds[1], STDOUT_FILENO) == -1)
-            die (SORT_FAILURE, errno, _("failed to reassign sort-undec STDOUT"));
+          /* replace the STDOUT of this process,
+             to the write-end pipe of 'undecorate' */
+          if (dup2 (sort_undec_fds[1], STDOUT_FILENO) == -1)
+            die (SORT_FAILURE, errno,
+                 _("failed to reassign sort-undec STDOUT"));
 
           char** sort_args = build_sort_process_args ();
           if (debug)
@@ -1025,11 +1117,11 @@ main (int argc, char **argv)
               char **p = sort_args;
               while (*p)
                 {
-                  fprintf(stderr,"sort: argv[] = '%s'\n", *p);
+                  dbg_printf ("sort: argv[] = '%s'", *p);
                   ++p;
                 }
             }
-          execvp("sort", sort_args);
+          execvp ("sort", sort_args);
 
           die (SORT_FAILURE, errno, _("failed to run the sort command"));
         }
@@ -1037,21 +1129,25 @@ main (int argc, char **argv)
         {
           /* This is the undecorate process */
           if (debug)
-            fprintf(stderr,"UNDECORATE child starting (pid %d)\n", getpid());
+            dbg_printf ("undecorate child starting");
 
-          if (close(dec_sort_fds[0])!=0)
+          if (close (dec_sort_fds[0])!=0)
             die (SORT_FAILURE, errno, _("failed to close dec-sort read-pipe"));
-          if (close(dec_sort_fds[1])!=0)
+          if (close (dec_sort_fds[1])!=0)
             die (SORT_FAILURE, errno, _("failed to close dec-sort write-pipe"));
-          if (close(sort_undec_fds[1])!=0)
-            die (SORT_FAILURE, errno, _("failed to close sort-undec write-pipe"));
+          if (close (sort_undec_fds[1])!=0)
+            die (SORT_FAILURE, errno,
+                 _("failed to close sort-undec write-pipe"));
 
           /* replace the STDIN of this process, to the read-end pipe */
-          if (dup2(sort_undec_fds[0], STDIN_FILENO) == -1)
+          if (dup2 (sort_undec_fds[0], STDIN_FILENO) == -1)
             die (SORT_FAILURE, errno, _("failed to reassign sort-undec STDIN"));
 
           /* undecorate from STDIN */
           do_undecorate (1, 0, NULL, num_decorate_fields);
+
+          if (debug)
+            dbg_printf ("undecorate child completed");
         }
     }
 
