@@ -76,6 +76,10 @@ struct operation_data operations[] =
   {STRING_SCALAR,  IGNORE_FIRST, STRING_RESULT},
   /* OP_MEAN */
   {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_GEOMEAN */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_HARMMEAN */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_MEDIAN */
   {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_QUARTILE_1 */
@@ -128,8 +132,12 @@ struct operation_data operations[] =
   {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_SHA1 */
   {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
+  /* OP_SHA224 */
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_SHA256 */
   {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
+  /* OP_SHA384 */
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_SHA512 */
   {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_P_COVARIANCE */
@@ -395,6 +403,14 @@ field_op_collect (struct fieldop *op,
       op->value += num_value;
       break;
 
+    case OP_GEOMEAN:
+      op->value += logl (num_value);
+      break;
+
+    case OP_HARMMEAN:
+      op->value += 1.0 / num_value;
+      break;
+
     case OP_COUNT:
       op->value++;
       break;
@@ -479,7 +495,9 @@ field_op_collect (struct fieldop *op,
     case OP_BASE64:
     case OP_MD5:
     case OP_SHA1:
+    case OP_SHA224:
     case OP_SHA256:
+    case OP_SHA384:
     case OP_SHA512:
     case OP_DIRNAME:
     case OP_BASENAME:
@@ -689,6 +707,8 @@ field_op_summarize_empty (struct fieldop *op)
   switch (op->op)                                /* LCOV_EXCL_BR_LINE */
     {
     case OP_MEAN:
+    case OP_GEOMEAN:
+    case OP_HARMMEAN:
     case OP_S_SKEWNESS:
     case OP_P_SKEWNESS:
     case OP_S_EXCESS_KURTOSIS:
@@ -755,7 +775,9 @@ field_op_summarize_empty (struct fieldop *op)
     case OP_DEBASE64:
     case OP_MD5:
     case OP_SHA1:
+    case OP_SHA224:
     case OP_SHA256:
+    case OP_SHA384:
     case OP_SHA512:
     case OP_DIRNAME:
     case OP_BASENAME:
@@ -801,6 +823,14 @@ field_op_summarize (struct fieldop *op)
       numeric_result = op->value / op->count;
       break;
 
+    case OP_GEOMEAN:
+      numeric_result = expl (op->value / op->count);
+      break;
+
+    case OP_HARMMEAN:
+      numeric_result = op->count / op->value;
+      break;
+
     case OP_SUM:
     case OP_COUNT:
     case OP_MIN:
@@ -985,11 +1015,21 @@ field_op_summarize (struct fieldop *op)
       field_op_to_hex (op, tmpbuf, 20);
       break;
 
+    case OP_SHA224:
+      sha224_buffer (op->str_buf, op->str_buf_used-1, tmpbuf);
+      field_op_to_hex (op, tmpbuf, 28);
+      break;
+
     case OP_SHA256:
       sha256_buffer (op->str_buf, op->str_buf_used-1, tmpbuf);
       field_op_to_hex (op, tmpbuf, 32);
       break;
 
+    case OP_SHA384:
+      sha384_buffer (op->str_buf, op->str_buf_used-1, tmpbuf);
+      field_op_to_hex (op, tmpbuf, 48);
+      break;
+
     case OP_SHA512:
       sha512_buffer (op->str_buf, op->str_buf_used-1, tmpbuf);
       field_op_to_hex (op, tmpbuf, 64);
