@@ -46,6 +46,8 @@ struct field_operation_definition field_operations[] =
   {"last",    OP_LAST,    MODE_GROUPBY},
   {"rand",    OP_RAND,    MODE_GROUPBY},
   {"mean",    OP_MEAN,    MODE_GROUPBY},
+  {"geomean", OP_GEOMEAN, MODE_GROUPBY},
+  {"harmmean", OP_HARMMEAN, MODE_GROUPBY},
   {"median",  OP_MEDIAN,  MODE_GROUPBY},
   {"q1",      OP_QUARTILE_1, MODE_GROUPBY},
   {"q3",      OP_QUARTILE_3, MODE_GROUPBY},
@@ -72,7 +74,9 @@ struct field_operation_definition field_operations[] =
   {"debase64",OP_DEBASE64,MODE_PER_LINE},
   {"md5",     OP_MD5,     MODE_PER_LINE},
   {"sha1",    OP_SHA1,    MODE_PER_LINE},
+  {"sha224",  OP_SHA224,  MODE_PER_LINE},
   {"sha256",  OP_SHA256,  MODE_PER_LINE},
+  {"sha384",  OP_SHA384,  MODE_PER_LINE},
   {"sha512",  OP_SHA512,  MODE_PER_LINE},
   {"dirname", OP_DIRNAME, MODE_PER_LINE},
   {"basename",OP_BASENAME,MODE_PER_LINE},
